#import <unistd.h>
#import "SpaceShipIDView.h"

#define	MAXPOINTS	20
#define	LABELSIZE	32
#define	FONTSCALE	20

@implementation SpaceShipIDView

- (id) initWithFrame:(NSRect) frame
	  isPreview:(BOOL) isPreview
{
	self = [super initWithFrame:frame
			  isPreview:isPreview];

	if (self != nil) {
		[self setAnimationTimeInterval:5.0];
		
		srandom(time((time_t *) NULL) * getpid());
		
		labelAttributes = [NSDictionary dictionaryWithObjectsAndKeys:
			[NSFont fontWithName:@"Courier"
					size:frame.size.height/FONTSCALE],
			NSFontAttributeName,
			[NSColor yellowColor],
			NSForegroundColorAttributeName,
			nil];
		
		[labelAttributes retain];
		}

    return(self);
}

- (void) startAnimation
{
	[super startAnimation];
}

- (void) stopAnimation
{
	[super stopAnimation];
}

- (void) drawRect:(NSRect) rect
{
	int		point1;			// first point
	int		point2;			// second point
	int		npoints;		// number of points
	char		labelbuf[LABELSIZE];	// label buffer
	NSRect		bounds;			// frame bounds
	NSSize		labelSize;		// label size
	NSPoint		labelPoint;		// label position
	NSPoint		points[MAXPOINTS];	// points array
	NSString *	label;			// label string
	NSBezierPath *	path;			// path to draw
	
	bounds       = [self bounds];
	npoints      = (random() % 8) + 3;
	path         = [NSBezierPath new];
	label        = [NSString stringWithUTF8String:
			[self makelabel:labelbuf]];
	labelSize    = [label sizeWithAttributes:labelAttributes];
	labelPoint.x = bounds.origin.x +
			((bounds.size.width - labelSize.width) / 2);
	labelPoint.y = bounds.origin.y + bounds.size.height / FONTSCALE;
	
	[path setLineWidth:2.0];
	
	for (point1 = 0; point1 < npoints; point1++) {
		points[point1].x = SSRandomFloatBetween(NSMinX(bounds),
							NSMaxX(bounds));
		points[point1].y = SSRandomFloatBetween(NSMinY(bounds) +
							bounds.size.height / (FONTSCALE / 2),
							NSMaxY(bounds));
		}
	
	for (point1 = 0; point1 < npoints; point1++) {
		for (point2 = 0; point2 < npoints; point2++) {
			if ((point1 != point2) &&
			    ((random() % 4) < 2)) {
				[path moveToPoint:points[point1]];
				[path lineToPoint:points[point2]];
				}
			}
		}
	
	[[NSColor blackColor] set];
	[NSBezierPath fillRect:bounds];
	
	[[NSColor greenColor] set];
	[path stroke];
	
	[label drawAtPoint:labelPoint
	    withAttributes:labelAttributes];
}

- (void) animateOneFrame
{
	[self setNeedsDisplay:YES];
}

- (BOOL) hasConfigureSheet
{
    return(NO);
}

- (NSWindow *) configureSheet
{
    return(nil);
}

- (BOOL) isOpaque
{
	// keep Cocoa from redrawing the superview all the time
	
	return(YES);
}

- (char *) makelabel:(char *) buf
{
	int	n;	// number of characters in section
	char *	bp;	// buffer pointer
	
	bp = buf;
	n  = random() % 2 + 2;
	
	while (n--) {
		*bp++ = [self randomlet];
		}
	
	n = random() % 4;
	
	if (n != 0) {
		*bp++ = '-';
		
		while (n--) {
			*bp++ = [self randomdig];
			}
		}
	
	n = random() % 2;
	
	if (n != 0) {
		*bp++ = '/';
		*bp++ = [self randomchar];
		}
	
	*bp++ = '\0';
	
	return(buf);
}

- (char) randomlet
{
	return((random() % 26) + 'A');
}

- (char) randomdig
{
	return((random() % 10) + '0');
}

- (char) randomchar
{
	int	c;	// which character
	
	c = random() % 36;
	
	if (c < 10) {
		return(c + '0');
		}
	
	return((c - 10) + 'A');
}

- (void) dealloc
{
	[labelAttributes release];
	
	[super dealloc];
}

@end
